// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.math.*;


/**
 * Extension of the ECashGen class which adds specific methods for a bank. 
 * @author Michel Deriaz
 * @see ECashGen
 * @see ECashBuyer
 * @see ECashSeller
 */
public class ECashBank extends ECashGen implements Runnable {
  private ECashBankListener listener;
  private KeyRing keyRing;
  private UsedNotesList usedNotesList;
  
  
  /**
   * Creates a ECashBank and loads the constants with the Const class.
   * @param listener the class that implements the ECashBankListener
   * interface
   */   
  public ECashBank(ECashBankListener listener) {
    this.listener = listener;
    Const.load("ECash");
    usedNotesList = new UsedNotesList();
  }
  
  
  /**
   * Returns an ECashBankState object. The idea is to write this object on
   * a hard-disk (using Tools.writeObject()) and retrieve it through the
   * setState() method next time the program is loaded (using
   * Tools.readObject()).
   * @return the current state
   * @see ECashBankState
   */   
  public ECashBankState getState() {
    return new ECashBankState(keyRing, usedNotesList);
  }
  

  /**
   * Initializes the class with the specified state.
   * @param state the state
   * @see #getState
   * @see ECashBankState
   */   
  public void setState(ECashBankState state) {
    keyRing = state.getKeyRing();
    usedNotesList = state.getUsedNotesList();
  }
  
    
  /**
   * Creates a new pair of keys.
   */    
  public void genKey() {
    keyRing = new KeyRing();
  }
  

  /**
   * Starts the server of the bank, in order to be ready to answer clients
   * requests.
   * @see ECashGen#stopServer
   */
  public void startServer() {
    Thread runner = new Thread(this);
    runner.start();
  }
  
  
  public void run() {
    if (!handshakeServer(BANK_PORT)) return;
    String op = receiveCrypted();
    if (op.equals(OP_GET_KEY)) opGetKey();
    if (op.equals(OP_BUY)) listener.report(opBuy());
    if (op.equals(OP_DEPOSIT)) listener.report(opDeposit());
    close();
    startServer();
  }
  
  
  private void opGetKey() {
    if (keyRing == null) return;
    sendCrypted(keyRing.getPublic().toString());
    sendCrypted(TRANS_OK);  
  }
      
    
  private BuyReport opBuy() {
    BuyReport buyReport = new BuyReport();
    String accountNumber = receiveCrypted();
    String password = receiveCrypted();
    buyReport.setAccountNumber(accountNumber);
    if (!listener.checkPassword(accountNumber, password)) {
      buyReport.setResult(CHEAT_WRONG_PASSWORD);
      return buyReport;
    }
    int nbNotes = Integer.parseInt(receiveCrypted());
    for (int i = 0; i < nbNotes; i++) {
      EnvelopesList envelopesList = new EnvelopesList(receiveCrypted());
      BankChoice bankChoice = new BankChoice(Const.NB_ENVELOPE);
      sendCrypted(bankChoice.toString());
      
      RevealedNotesList revealedNotesList = new RevealedNotesList(receiveCrypted());
      String amount = revealedNotesList.getAmount();
      if (amount.equals("-1")) {
        buyReport.setResult(CHEAT_AMOUNTS_NOT_ALL_THE_SAME);
        return buyReport;
      }
      buyReport.addAmount(amount);
      if (!envelopesList.checkRevealed(revealedNotesList, bankChoice, accountNumber, keyRing.getPublic())) {
        buyReport.setResult(CHEAT_WRONG_REVEALED_NOTES);
        return buyReport;
      }
      BigInteger signedEnvelope = Tools.signEnvelope(envelopesList.get(bankChoice.toInt()), keyRing);
      sendCrypted(signedEnvelope.toString());
      
      sendCrypted(TRANS_OK);
    }
    double currentAmount = Double.parseDouble(listener.getAmount(accountNumber));
    double diff = Double.parseDouble(buyReport.computeSum());
    sendCrypted("" + (currentAmount-diff));    
    sendCrypted(TRANS_OK);    
    buyReport.setResult(OK);
    return buyReport;
  }
  
  
  private DepositReport opDeposit() {
    DepositReport depositReport = new DepositReport();
    String accountNumber = receiveCrypted();
    String password = receiveCrypted();
    depositReport.setAccountNumber(accountNumber);
    if (!listener.checkPassword(accountNumber, password)) {
      depositReport.setResult(CHEAT_WRONG_PASSWORD);
      return depositReport;
    }
    int nbNotes = Integer.parseInt(receiveCrypted());
    UsedNotesList usedNotesListCopy = usedNotesList.copy();
    for (int i = 0; i < nbNotes; i++) {
      Note note = new Note(receiveCrypted());
      BigInteger challengeRIS = new BigInteger(receiveCrypted());                
      RISPart risPart = new RISPart(receiveCrypted());
      if (!note.checkSignature(keyRing.getPublic())) {
        depositReport.setResult(CHEAT_WRONG_SIGNATURE);
        return depositReport;
      }
      if (!note.checkRIS(challengeRIS, risPart)) {
        depositReport.setResult(CHEAT_WRONG_RIS);
        return depositReport;
      }
      int answer = usedNotesListCopy.check(note.getBigM(), risPart.getValues());
      if (answer == -1) {
        depositReport.setResult(CHEAT_NOTE_REUSED_BY_SELLER);
        return depositReport;
      }
      if (answer > 0) {
        depositReport.setResult(CHEAT_NOTE_REUSED_BY_BUYER);
        depositReport.setCheaterAccountNumber("" + answer);
        return depositReport;
      }
      usedNotesListCopy.add(note.getBigM(), risPart.getValues());
      depositReport.addAmount(note.getAmount());
      sendCrypted(TRANS_OK);
    }
    double currentAmount = Double.parseDouble(listener.getAmount(accountNumber));
    double diff = Double.parseDouble(depositReport.computeSum());
    sendCrypted("" + (currentAmount+diff));    
    sendCrypted(TRANS_OK);    
    usedNotesList = usedNotesListCopy;
    depositReport.setResult(OK);
    return depositReport;
  }
}