// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.math.*;
import java.util.*;


/**
 * Extension of the ECashGen class which adds specific methods for a
 * buyer. 
 * @author Michel Deriaz
 * @see ECashGen
 * @see ECashBank
 * @see ECashSeller
 */
public class ECashBuyer extends ECashGen {
  private ECashBuyerListener listener;  
  private BuyerPurse buyerPurse;
  

  /**
   * Creates an ECashBuyer and loads the constants with the Const class.
   * @param listener the class that implements the ECashBuyerListener
   * interface
   */   
  public ECashBuyer(ECashBuyerListener listener) {
    this.listener = listener;
    Const.load("ECash");
    buyerPurse = new BuyerPurse();
  }
  

  /**
   * Returns an ECashBuyerState object. The idea is to write this object on
   * a hard-disk (using Tools.writeObject()) and retrieve it through the
   * setState() method next time the program is loaded (using
   * Tools.readObject()).
   * @return the current state
   * @see ECashBuyerState
   */     
  public ECashBuyerState getState() {
    return new ECashBuyerState(buyerPurse, pubKey);
  }
  

  /**
   * Initializes the class with the specified state.
   * @param state the state
   * @see #getState
   * @see ECashBuyerState
   */     
  public void setState(ECashBuyerState state) {
    buyerPurse = state.getBuyerPurse();
    pubKey = state.getPubKey();
  }
    

  /**
   * Returns the list of all the amounts of the different notes. Each
   * entry of this Vector is a String.
   * @return the list of all the amounts from the different notes
   */   
  public Vector getAmountList() {
    return buyerPurse.getAmountList();
  }
  
  
  /**
   * Creates some notes and ask the bank to sign them.
   * @param bankAddress the address of the bank, for example 
   * "124.123.132.233"
   * @param bankAccount the account number
   * @param password the password of the account
   * @param notesExp the notes expression describing which notes must be 
   * created.
   * @return a report describing the operation
   * @see CreditDebitReport
   */   
  public CreditDebitReport buy(String bankAddress, String bankAccount, char[] password, String notesExp) {
    CreditDebitReport report = new CreditDebitReport();
    Vector values;
    if ((notesExp != null) && (!notesExp.equals(""))) values = Tools.notesExpToValues(notesExp);
    else values = new Vector();    
    if (values == null) {
      report.setResult(NOTES_EXP_SYNTAX_ERROR);
      return report;
    }
    try {
      boolean ok = handshakeClient(bankAddress, BANK_PORT);
      if (!ok) {
        report.setResult(SERVER_NOT_READY);
        return report;
      }
      sendCrypted(OP_BUY);
      sendCrypted(bankAccount);
      sendCrypted(password);
      buyerPurse.clear();
      sendCrypted("" + values.size());
      for (int i = 0; i < values.size(); i++) {  
        String amount = (String)values.get(i);
        FullNotesList fullNotesList = new FullNotesList(Const.NB_ENVELOPE, amount, bankAccount, pubKey);
        EnvelopesList envelopesList = fullNotesList.getEnvelopesList();
        sendCrypted(envelopesList.toString());
        BankChoice bankChoice = new BankChoice(receiveCrypted());
        RevealedNotesList revealedNotesList = new RevealedNotesList(fullNotesList, bankChoice);
        sendCrypted(revealedNotesList.toString());
        BigInteger signedEnvelope = new BigInteger(receiveCrypted());
        BigInteger bigM = fullNotesList.get(bankChoice.toInt()).getBigM();
        BigInteger signedNote = Tools.unblind(signedEnvelope, fullNotesList, bankChoice, pubKey);
        Note note = new Note(bigM, signedNote);
        if (!note.checkSignature(pubKey)) {
          report.setResult(CHEAT_WRONG_SIGNATURE);
          return report;
        }
        if (!receiveCrypted().equals(TRANS_OK)) throw new Exception();
        buyerPurse.add(note, fullNotesList, bankChoice);
      }
      report.setCurrentAmount(receiveCrypted());
      if (!receiveCrypted().equals(TRANS_OK)) throw new Exception();      
      close();
    }
    catch(Exception e) {
      report.setResult(CONNECTION_CLOSED_BY_SERVER);
      return report;
    }
    buyerPurse.addCommit();
    listener.buyerPurseChanged();
    report.setResult(OK);
    return report;
  }
  
 
  /**
   * Sends some notes to a seller. Returns an int that represents how the
   * operation succeeded. Value can be
   * <br>ECashGen.OK, 
   * <br>ECashGen.NOTES_EXP_SYNTAX_ERROR,
   * <br>ECashGen.NOTES_NOT_AVAILABLE,
   * <br>ECashGen.SERVER_NOT_READY or
   * <br>ECashGen.CONNECTION_CLOSED_BY_SERVER.
   *
   * @param sellerAddress the address of the seller, for example 
   * "124.123.132.233"
   * @param nickname the name which will be sended, allowing the seller to
   * identify the paiement.
   * @param notesExp the notes expression describing which notes must be 
   * sended.
   * @return an integer describing the success of the operation
   * @see ECashGen
   */     
  public int pay(String sellerAddress, String nickname, String notesExp) {
    Vector values = Tools.notesExpToValues(notesExp);
    if (values == null) return NOTES_EXP_SYNTAX_ERROR;
    if (!checkIfNotesAvailable(values, buyerPurse.getAmountList())) return NOTES_NOT_AVAILABLE;
        
    try {
      boolean ok = handshakeClient(sellerAddress, SELLER_PORT);
      if (!ok) return SERVER_NOT_READY;
      sendCrypted(OP_PAY);
      sendCrypted(nickname);
      buyerPurse.clear();
      sendCrypted("" + values.size());
      buyerPurse.initFindAmount();
      for (int i = 0; i < values.size(); i++) {
        int index = buyerPurse.findAmount((String)values.get(i));
        Note note = buyerPurse.getNote(index);
        sendCrypted(note.toString());
        BigInteger challengeRIS = new BigInteger(receiveCrypted());
        FullNotesList fullNotesList = buyerPurse.getFullNotesList(index);
        BankChoice bankChoice = buyerPurse.getBankChoice(index);            
        RISPart risPart = new RISPart(fullNotesList, bankChoice, challengeRIS);
        sendCrypted(risPart.toString());
        if (!receiveCrypted().equals(TRANS_OK)) throw new Exception();
        buyerPurse.remove(index);
      }
      close();
    }
    catch(Exception e) {
      return CONNECTION_CLOSED_BY_SERVER;
    }
    buyerPurse.removeCommit();
    listener.buyerPurseChanged();
    return OK;
  }      
}