// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.math.*;
import javax.crypto.*;
import java.security.*;
import javax.crypto.spec.*;
import java.security.interfaces.*;


/**
 * Generic class for ECashBank, ECashBuyer and ECashSeller. 
 * @author Michel Deriaz
 * @see ECashBank
 * @see ECashBuyer
 * @see ECashSeller
 */
public class ECashGen {
  
  
  /**
   * This code, transmitted through the network, informs the partner of the
   * current operation: The sender wants to buy some notes. 
   */ 
  public static final String OP_BUY = "Operation buy";
  
  
  /**
   * This code, transmitted through the network, informs the partner of the
   * current operation: The sender wants to give some notes to the seller.
   */   
  public static final String OP_PAY = "Operation pay";
    

  /**
   * This code, transmitted through the network, informs the partner of the
   * current operation: The sender wants to deposit some notes at the bank.
   */   
  public static final String OP_DEPOSIT = "Operation deposit";
  
  
  /**
   * This code, transmitted through the network, informs the partner of the
   * current operation: The sender wants the public key of the bank.
   */   
  public static final String OP_GET_KEY = "Operation get public key of the bank";
  
  
  /**
   * This code, transmitted through the network, informs the partner that 
   * the last set of transactions was successful.
   */   
  public static final String TRANS_OK = "Transistion completed successfully";
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the operation (buy, deposit, ...) was correct and no 
   * cheating has been detected (like re-using a note twice, wrong 
   * signature, ...).
   */   
  public static final int OK = 0;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the password (or the bank account) is not correct.
   */   
  public static final int CHEAT_WRONG_PASSWORD = 1;


  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the amounts of the revealed notes are not all the same.
   * This can occur when a client tries to make the bank sign a note with
   * a bigger amount than revealed in the other notes.
   */   
  public static final int CHEAT_AMOUNTS_NOT_ALL_THE_SAME = 2;
  

  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the revealed notes are not correct.
   */     
  public static final int CHEAT_WRONG_REVEALED_NOTES = 3;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the signature is not correct (or that the signed part
   * doesn't correspond to the unsigned one).
   */     
  public static final int CHEAT_WRONG_SIGNATURE = 4;
    

  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the RIS is not correct.
   */      
  public static final int CHEAT_WRONG_RIS = 5;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the seller re-used twice the same note. This occurs 
   * when the bank get twice the same note with the same RIS. 
   */    
  public static final int CHEAT_NOTE_REUSED_BY_SELLER = 6;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the buyer re-used twice the same note. This occurs 
   * when the bank get twice the same note but with differents valids RIS. 
   */    
  public static final int CHEAT_NOTE_REUSED_BY_BUYER = 7;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the connection was closed by the server. This can 
   * append either because of a technical problem, or because the client
   * cheats. The term "cheat" covers unvoluntary cheating, like providing
   * a wrong password or a wrong account number, or voluntary cheating, 
   * like re-using several time the same note.
   */      
  public static final int CONNECTION_CLOSED_BY_SERVER = 11;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the server (bank or seller) is not ready yet. One 
   * reason could simply be that the server is already in connection with
   * someone else.
   */        
  public static final int SERVER_NOT_READY = 12;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the public key is not available.
   */     
  public static final int KEY_NOT_AVAILABLE = 13;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the notes expression carries one or several syntax 
   * errors.
   */       
  public static final int NOTES_EXP_SYNTAX_ERROR = 14;
  
  
  /**
   * This code, used locally in reports or as function returned value, 
   * indicates that the requested notes (for example during a paying 
   * operation) are not availables. 
   */     
  public static final int NOTES_NOT_AVAILABLE = 15;
  
  
  /**
   * This constant defines on which port the bank is waiting for requests.
   */   
  public static final int BANK_PORT = 10000;
  
  
  /**
   * This constant defines on which port the seller is waiting for
   * requests.
   */     
  public static final int SELLER_PORT = 10001;


  protected PubKey pubKey;

  private static final int RSA_KEY_SIZE = 512;  // [512..2048]
  private static final int BF_KEY_SIZE = 127;  
    // [32..127] or [32..448] without export restrictions    
          
  private Socket s;
  private ServerSocket ss;
  private Cipher cipher;
  private SecretKeySpec secretKey;
  private ObjectOutputStream out;   
  private ObjectInputStream in;  
  private boolean serverStopped;


  protected ECashGen() {
  }
  

  /**
   * Returns true if the public key is available, false otherwise.
   * @return a boolean indicating whether the public key is available or
   * not
   */ 
  public boolean hasBankKey() {
    return (pubKey != null);
  }
  

  /**
   * Returns the public key of the bank.
   * @return the public key of the bank
   *
   */   
  public PubKey getKey(){
    return pubKey;
  }
  
  
  /**
   * Connects to the bank and asks its public key. Returns a code that 
   * indicate how the operation succeeded. The code can be
   * <br>ECashGen.OK,
   * <br>ECashGen.SERVER_NOT_READY or
   * <br>ECashGen.KEY_NOT_AVAILABLE.
   *
   * @param bankAddress the address of the bank, for example
   * "123.123.33.123"
   * @return an integer describing the success of the operation 
   */   
  public int getKey(String bankAddress) {
    try {
      boolean ok = handshakeClient(bankAddress, BANK_PORT);
      if (!ok) return SERVER_NOT_READY;
      sendCrypted(OP_GET_KEY);
      pubKey = new PubKey(receiveCrypted());    
      if (!receiveCrypted().equals(TRANS_OK)) throw new Exception();
      close();
    }
    catch(Exception e) {
      return KEY_NOT_AVAILABLE;
    }
    return OK;
  }
      
  
  /**
   * Sets the public key.
   * @param pubKey the public key
   */   
  public void setKey(PubKey pubKey) {
    this.pubKey = pubKey;
  }
  

  /**
   * Returns the regular expression that is used to define the syntax of 
   * the notes expression. It can be used to control users inputs.
   * @return the regular expression defining the syntax of a notes
   * expressions
   */   
  public String getREGEXP() {
    return Const.REGEXP;
  }
  

  /**
   * Stops the server. This method can be used by the bank or a seller to 
   * avoid any connection from clientxs
   */ 
  public void stopServer() {
    serverStopped = true;
    if (ss != null) {
      try {
        ss.close();
      }
      catch (IOException e) {}    
    }
  }    


  /**
   * Returns true if the amounts list contains all the notes found in the
   * notes expression items list, false otherwise. The amount list is a 
   * vector of String, and each item is an amount of an available note. The
   * notes expression items vector is also a vector of String, and can be 
   * considered as a splitted notes expression.
   * <br><strong>Warning:</strong> The amounts list will be modified by this 
   * method.
   * @param notesExpItems a vector of String that represents a splitted 
   * notes expression
   * @param amountsList a vector of String that represents all the
   * availables notes
   * @return a boolean indicating whether the requested notes are
   * availables or not
   */     
  public boolean checkIfNotesAvailable(Vector notesExpItems, Vector amountList) {
    for (int i = 0; i < notesExpItems.size(); i++) {
      if (amountList.contains(notesExpItems.get(i))) amountList.remove(notesExpItems.get(i));
      else return false;
    }
    return true;
  }
  

  /**
   * Connects to a server and constructs a secure crypted channel to it.
   * Returns true if the operation succeed, false otherwise.
   * @param serverAddress the address of the server
   * @param port the port of the server
   * @return a boolean indicating whether the operation succeeded or not
   */   
  public boolean handshakeClient(String serverAddress, int port) {  
    try {
      s = new Socket(serverAddress, port);
      System.out.println("Connection with " + s.getInetAddress() + " on port " + s.getPort());
      out = new ObjectOutputStream(s.getOutputStream());   
      in = new ObjectInputStream(s.getInputStream());
      KeyPair kp = generateRSAKey();
      RSAPublicKey publicKey = (RSAPublicKey)kp.getPublic();
      RSAPrivateKey privateKey = (RSAPrivateKey)kp.getPrivate();
      send(publicKey);
      BigInteger encryptedKey = (BigInteger)in.readObject();     
      byte[] key = decryptRSA(encryptedKey, privateKey);
      secretKey = new SecretKeySpec(key, "Blowfish");
      cipher = Cipher.getInstance("Blowfish");
    }
    catch (Exception e) {
      return false;
    }
    return true;
  }


  /**
   * Connects to a client and constructs a secure crypted channel to it.
   * Returns true if the operation succeed, false otherwise.
   * @param port the port of the server
   * @return a boolean indicating whether the operation succeeded or not
   */   
  public boolean handshakeServer(int port) {
    try {
      System.out.println ("\nMy address: " + InetAddress.getLocalHost().toString());
      serverStopped = false;
      ss = new ServerSocket(port); 
      System.out.println("Server listening on port " + port);
      s = ss.accept();
      ss.close();
      System.out.println("Connection with " + s.getInetAddress());
      out = new ObjectOutputStream(s.getOutputStream());
      in = new ObjectInputStream(s.getInputStream());
      RSAPublicKey publicKey = (RSAPublicKey)in.readObject();
      BigInteger keyBI = new BigInteger(BF_KEY_SIZE, new SecureRandom());
      BigInteger encyptedKey = encryptRSA(keyBI, publicKey);
      out.writeObject(encyptedKey);
      out.flush();
      byte[] key = keyBI.toByteArray();
      secretKey = new SecretKeySpec(key, "Blowfish");
      cipher = Cipher.getInstance("Blowfish");
    }
    catch (Exception e) {
      if (serverStopped) return false;
      else System.out.println(e);
    }
    return true;
  }


  /**
   * Sends a message over a secured crypted channel. The handshakeClient()
   * or the handshakeServer() methods can be used to set up such a channel.
   * @param message the message to crypt and to send 
   * @see #handshakeClient
   * @see #handshakeServer
   */ 
  public void sendCrypted(String message) {
    try {
      byte[] ciphertext = encryptBF(message, cipher, secretKey);
      out.writeObject(ciphertext);
      out.flush();
    }
    catch (IOException e) {
      System.out.println(e);
    }
  }
  
  /**
   * Sends a array of char, likely a password, over a secured crypted
   * channel. The handshakeClient() or the handshakeServer() methods can be
   * used to set up such a channel.
   * @param message the message to crypt and to send
   * @see #handshakeClient
   * @see #handshakeServer
   */   
  public void sendCrypted(char[] message) {
    sendCrypted(String.valueOf(message));
  }


  /**
   * Receives a message from a secured crypted channel. The
   * handshakeClient() or the handshakeServer() methods can be used to set
   * up such a channel. 
   * <br><strong>Note:</strong> This method will block until an input is
   * available.
   * @return the message, as a plaintext
   * @see #handshakeClient
   * @see #handshakeServer   
   */   
  public String receiveCrypted() {    
    String result = null;
    try {
      byte[] input = (byte[])in.readObject();
      result = decryptBF(input, cipher, secretKey);
    }
    catch (Exception e) {
      System.out.println(e);
    } 
    return result;
  }
                

  /**
   * Closes the socket.
   */ 
  public void close() {
    try {
      s.close();
    }
    catch (Exception e) {
      System.out.println(e);
    }    
  }


  private KeyPair generateRSAKey() { 
    KeyPairGenerator keyPairGen = null;
    try {
      keyPairGen = KeyPairGenerator.getInstance("RSA");
    }
    catch(NoSuchAlgorithmException e) {
      System.out.println(e);
      return null;
    }      
    keyPairGen.initialize(RSA_KEY_SIZE, new SecureRandom());
    return keyPairGen.generateKeyPair();      
  }
  

  private BigInteger encryptRSA(BigInteger plain, RSAPublicKey publicKey) {
    return plain.modPow(publicKey.getPublicExponent(), publicKey.getModulus());
  }
  
    
  private byte[] decryptRSA(BigInteger cipher, RSAPrivateKey privateKey) {
    return cipher.modPow(privateKey.getPrivateExponent(), privateKey.getModulus()).toByteArray();
  }  
  

  private byte[] encryptBF(String input, Cipher cipher, SecretKey secretKey) {
    byte[] result = null;
    try {
      cipher.init(Cipher.ENCRYPT_MODE, secretKey);
      result = cipher.doFinal(input.getBytes("ISO-8859-1"));
    }
    catch (Exception e) {
      System.out.println(e);
    }
    return result;
  }  
  
    
  private String decryptBF(byte[] input, Cipher cipher, SecretKey secretKey) {
    String result = null;
    try {
      cipher.init(Cipher.DECRYPT_MODE, secretKey);
      result = new String(cipher.doFinal(input), "ISO-8859-1");
    }
    catch (Exception e) {
      System.out.println(e);
    }
    return result;
  }
  
  
  private void send(RSAPublicKey publicKey) {
    try {
      out.writeObject(publicKey);            
      out.flush(); 
    }
    catch (IOException e) {
      System.out.println(e);
    }
  }
}