// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.math.*;
import java.util.*;
import java.security.*;


/**
 * Extension of the ECashGen class which adds specific methods for a
 * seller. 
 * @author Michel Deriaz
 * @see ECashGen
 * @see ECashBank
 * @see ECashBuyer
 */
public class ECashSeller extends ECashGen implements Runnable {
  private ECashSellerListener listener;
  private SellerPurse sellerPurse;


  /**
   * Creates an ECashSeller and loads the constants with the Const class.
   * @param listener the class that implements the ECashSellerListener
   * interface
   */   
  public ECashSeller(ECashSellerListener listener) {
    this.listener = listener;
    Const.load("ECash");
    sellerPurse = new SellerPurse();    
  }
  
  
  /**
   * Returns an ECashSellerState object. The idea is to write this object
   * on a hard-disk (using Tools.writeObject()) and retrieve it through the
   * setState() method next time the program is loaded (using
   * Tools.readObject()).
   * @return the current state
   * @see ECashSellerState
   */       
  public ECashSellerState getState() {
    return new ECashSellerState(sellerPurse, pubKey);
  }
  

  /**
   * Initializes the class with the specified state.
   * @param state the state
   * @see #getState
   * @see ECashSellerState
   */     
  public void setState(ECashSellerState state) {
    sellerPurse = state.getSellerPurse();
    pubKey = state.getPubKey();
  }
    

  /**
   * Returns the list of all the amounts of the different notes. Each
   * entry of this Vector is a String.
   * @return the list of all the amounts from the different notes
   */       
  public Vector getAmountList() {
    return sellerPurse.getAmountList();
  }
   
    
  /**
   * Starts the server of the seller, in order to be ready to answer
   * buyers requests.
   * @see ECashGen#stopServer
   */
  public void startServer() {
    Thread runner = new Thread(this);
    runner.start();
  }
  

  /**
   * Deposits some notes at the bank.
   * @param bankAddress the address of the bank, for example 
   * "124.123.132.233"
   * @param bankAccount the account number
   * @param password the password of the account
   * @param notesExp the notes expression describing which notes must be 
   * deposited.
   * @return a report describing the operation
   * @see CreditDebitReport
   */   
  public CreditDebitReport deposit(String bankAddress, String bankAccount, char[] password, String notesExp) {
    CreditDebitReport report = new CreditDebitReport();
    Vector values;
    if ((notesExp != null) && (!notesExp.equals(""))) values = Tools.notesExpToValues(notesExp);
    else values = new Vector();    
    if (values == null) {
      report.setResult(NOTES_EXP_SYNTAX_ERROR);
      return report;
    }
    if (!checkIfNotesAvailable(values, sellerPurse.getAmountList())) {
      report.setResult(NOTES_NOT_AVAILABLE);
      return report;
    }
    try {
      boolean ok = handshakeClient(bankAddress, BANK_PORT);
      if (!ok) {
        report.setResult(SERVER_NOT_READY);
        return report;
      }
      sendCrypted(OP_DEPOSIT);
      sendCrypted(bankAccount);
      sendCrypted(password);
      sellerPurse.clear();
      sendCrypted("" + values.size());
      sellerPurse.initFindAmount();
      for (int i = 0; i < values.size(); i++) {
        int index = sellerPurse.findAmount((String)values.get(i));
        Note note = sellerPurse.getNote(index);
        BigInteger challengeRIS = sellerPurse.getChallengeRIS(index);
        RISPart risPart = sellerPurse.getRISPart(index);        
        sendCrypted(note.toString());
        sendCrypted(challengeRIS.toString());
        sendCrypted(risPart.toString());
        if (!receiveCrypted().equals(TRANS_OK)) throw new Exception();
        sellerPurse.remove(index);
      }
      report.setCurrentAmount(receiveCrypted());
      if (!receiveCrypted().equals(TRANS_OK)) throw new Exception();      
      close();
    }
    catch(Exception e) {
      report.setResult(CONNECTION_CLOSED_BY_SERVER);
      return report;
    }
    sellerPurse.removeCommit();  
    listener.sellerPurseChanged();     
    report.setResult(OK);
    return report;
  }
  
      
  public void run() {
    if (!handshakeServer(SELLER_PORT)) return;
    String op = receiveCrypted();
    if (op.equals(OP_PAY)) listener.report(opPay());
    close();
  }
  
  
  private PayReport opPay() {
    PayReport payReport = new PayReport();    
    String buyerNickname = receiveCrypted();
    payReport.setBuyerNickname(buyerNickname);
    int nbNotes = Integer.parseInt(receiveCrypted());
    for (int i = 0; i < nbNotes; i++) {    
      Note note = new Note(receiveCrypted());
      if (!note.checkSignature(pubKey)) {
        payReport.setResult(CHEAT_WRONG_SIGNATURE);
        return payReport;
      }
      BigInteger challengeRIS = new BigInteger(Const.NB_RIS, new SecureRandom());
      sendCrypted(challengeRIS.toString());
      RISPart risPart = new RISPart(receiveCrypted());
      if (!note.checkRIS(challengeRIS, risPart)) {
        payReport.setResult(CHEAT_WRONG_RIS);
        return payReport;
      }
      payReport.addAmount(note.getAmount());
      sendCrypted(TRANS_OK);
      sellerPurse.add(note, challengeRIS, risPart, buyerNickname);
    }
    sellerPurse.addCommit();
    payReport.setResult(OK);
    listener.sellerPurseChanged();
    return payReport;
  }
}