// Updates: 2003.06.13


package ecash.internet;


import ecash.kernel.*;
import java.util.*;


/**
 * A Report indicates how an operation succeeded, from the server point of
 * view. A BuyReport is created for the bank when a buyer buyes some
 * notes, a DepositReport is created for the bank when a seller deposits 
 * some notes and a PayReport is created for the seller when a buyer sends
 * him some notes. This abstract class contains all the common methods
 * used by its subclasses.
 * @author Michel Deriaz
 * @see ECashGen
 */
public abstract class Report {
  private String amounts = null;
  private int result = -1; 
  
  
  protected Report() {
  }
    
  
  /**
   * Returns a string that represents a list of all the amounts, separated
   * by a coma.
   * @return a string that represents a list of all the amounts, separated
   * by a coma
   */   
  public String getAmounts() {
    return amounts;
  }
  

  /**
   * Returns an int that represents how the operation succeeded.
   *
   * <p>For a buy operation (BuyReport), it can be
   * <br>ECashGen.OK, 
   * <br>ECashGen.CHEAT_WRONG_PASSWORD, 
   * <br>ECashGen.CHEAT_AMOUNTS_NOT_ALL_THE_SAME or
   * <br>ECashGen.CHEAT_WRONG_REVEALED_NOTES.
   *   
   * <p>For a deposit operation (DepositReport) it can be 
   * <br>ECashGen.OK,
   * <br>ECashGen.CHEAT_WRONG_PASSWORD,
   * <br>ECashGen.CHEAT_WRONG_SIGNATURE,
   * <br>ECashGen.CHEAT_WRONG_RIS,
   * <br>ECashGen.CHEAT_NOTE_REUSED_BY_SELLER or
   * <br>ECashGen.CHEAT_NOTE_REUSED_BY_BUYER.
   *
   * <p>For a pay operation (PayReport) it can be 
   * <br>ECashGen.OK,
   * <br>ECashGen.CHEAT_WRONG_SIGNATURE or
   * <br>ECashGen.CHEAT_WRONG_RIS.
   *    
   * @return an int that represents how the operation succeeded
   * @see ECashGen
   */ 
  public int getResult() {
    return result;
  }
 
  
  /**
   * Adds an amount to the amounts list.
   * @param s the amount to add
   */   
  public void addAmount(String s) {
    if (amounts == null) amounts = s;
    else amounts = amounts + ", " + s;
  }
  

  /**
   * Set the result that describes how the operation succeeded.
   * @param result an integer describing how the operation succeeded
   * @see #getResult
   */ 
  public void setResult(int result) {
    this.result = result;
  }
  
  
  /**
   * Computes and returns the sum of all the amounts found in the amounts
   * list.
   * @return the sum of all the amounts found in the amounts list
   */   
  public String computeSum() {
    if (amounts == null) return "0.0";
    String[] tockens = amounts.split(", ");
    double sum = 0.0;
    for (int i = 0; i < tockens.length; i++) {
      sum += Double.parseDouble(tockens[i]);
    }
    return "" + Tools.round(sum);
  }
  
  
  /**
   * Computes and returns the notes expression for the amounts list.
   * @return the notes expression for the amounts list
   */   
  public String computeNotesExp() {
    if (amounts == null) return "";
    Vector v = new Vector();
    String[] tockens = amounts.split(", ");
    for (int i = 0; i < tockens.length; i++) {
      v.add(tockens[i]);
    } 
    return Tools.valuesToNotesExp(v);
  }
}    