// Updates: 2003.06.13


package ecash.kernel;


import java.io.*;
import java.util.*;


/**
 * Contains all the notes and information used to build them. Calls to the
 * add() and the remove() methods will be taken into account only after a
 * call to addCommit() and removeCommit() respectively. The clear() method
 * will initialize this process. Each item of this list is a BuyerPurseUnit
 * object.
 * @author Michel Deriaz
 * @see BuyerPurseUnit
 */
public class BuyerPurse implements Serializable {
  private Vector v, vTmp, indexes;
  

  /**
   * Creates a BuyerPurse.
   */     
  public BuyerPurse() {
    v = new Vector();
    clear();
  }
  
  
  /**
   * Adds a new note and its building information.
   * @param note the note
   * @param fullNotesList all the information used to create the note, and 
   * also the other notes (the bank gets several notes and then choose 
   * wich one she will sign
   * @param bankChoice the note chosen by the bank
   * @see #addCommit
   */   
  public void add(Note note, FullNotesList fullNotesList, BankChoice bankChoice) {
    vTmp.add(new BuyerPurseUnit(note, fullNotesList, bankChoice));
  }
  

  /**
   * Let's take into effect all the calls made to the add() method, and 
   * then calls the clear() method.
   * @see #add
   */
  public void addCommit() {
    v.addAll(vTmp);
    clear();
  }
  
  
  /**
   * Removes a note.
   * @param index the index of the note to remove
   * @see #removeCommit
   */
  public void remove(int index) {
    vTmp.add(v.get(index));
  }
  

  /**
   * Let's take into effect all the calls made to the remove() method, and 
   * then calls the clear() method.
   * @see #remove
   */
  public void removeCommit() {
    v.removeAll(vTmp);
    clear();
  }  
  
  
  /**
   * Gets the specified note.
   * @param index the index in the list
   */  
  public Note getNote(int index) {
    return ((BuyerPurseUnit)v.get(index)).getNote();
  }
  

  /**
   * Gets the FullNotesList of the specified note.
   * @param index the index in the list
   */   
  public FullNotesList getFullNotesList(int index) {
    return ((BuyerPurseUnit)v.get(index)).getFullNotesList();
  }
  
  
  /**
   * Gets the BankChoice of the specified note.
   * @param index the index in the list
   */  
  public BankChoice getBankChoice(int index) {
    return ((BuyerPurseUnit)v.get(index)).getBankChoice();
  }
  

  /**
   * Returns the amounts list of the notes. Each entry of this Vector is a
   * String.
   * @return a vector containing the amounts of all the notes
   */
  public Vector getAmountList() {
    Vector result = new Vector();
    for(int i = 0; i < v.size(); i++) {
      BuyerPurseUnit bpu = (BuyerPurseUnit)v.get(i);
      result.add("" + bpu.getNote().getAmount());
    }
    return result;
  }
 
  
  /**
   * Returns the number of notes.
   * @return the number of notes
   */  
  public int size() {
    return v.size();
  }
  
  
  /**
   * Initializes the data for the add() and the remove() methods.
   */  
  public void clear() {
    vTmp = new Vector();
  }
  

  /**
   * Initializes the findAmount() method.
   */  
  public void initFindAmount() {
    indexes = new Vector();
  }
  
  
  /**
   * Returns the first index of a note with the specified amount. Then the 
   * note is "marked" in order to avoid to return the same note when the
   * method is called again with the same parameter. The initFindAmount()
   * will "unmark" all the notes.
   * @param amount the amount to find
   * @return the index in the list
   */  
  public int findAmount(String amount) {
    for (int i = 0; i < v.size(); i++) {
      if (indexes.contains("" + i)) continue;
      BuyerPurseUnit bpu = (BuyerPurseUnit)v.get(i);
      if (bpu.getNote().getAmount().equals(amount)) {
        indexes.add("" + i);
        return i;
      }
    }
    return -1;
  }
}