// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;
import java.util.*;


/**
 * The list of envelopes sended to the bank. The latter will then choose
 * one of them and return it signed to the client.
 * @author Michel Deriaz
 */  
public class EnvelopesList implements Serializable {
  private Vector v;


  /**
   * Constructs a new list.
   */ 
  public EnvelopesList() {
    v = new Vector();
  }
  
  
  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */  public EnvelopesList(String value) {
    String[] tockens = value.split("\n");
    v = new Vector();
    for (int i = 0; i < tockens.length; i++) {
      v.add(new BigInteger(tockens[i]));
    }
  }
  
  
  /**
   * Adds the specified envelope to the list.
   * @param envelope the envelope
   */   
  public void add(BigInteger envelope) {
    v.add(envelope);
  }

  
  /**
   * Returns the specified envelope.
   * @param index the index of the envelope
   * @return the envelope
   */   
  public BigInteger get(int index) {
    return (BigInteger)v.get(index);
  }


  /**
   * Returns true if the information used to build all the envelopes
   * except the one chosen by the bank is correct, false otherwise.
   * @param revealedNotesList information used to build the envelopes
   * (except for the one chosen by the bank)
   * @param bankChoice the choice of the bank (all the other envelopes must
   * be revealed)
   * @param id the identification of the client (bank account)
   * @param pubKey the public key of the bank
   * @return a boolean indicating whether the revealed information is 
   * correct or not
   */ 
  public boolean checkRevealed(RevealedNotesList revealedNotesList, BankChoice bankChoice, String id, PubKey pubKey) {
    boolean cheat = false;
    int i = -1;
    for (int j = 0; j < revealedNotesList.size(); j++) {
      i++;
      if (j == bankChoice.toInt()) i++;
      BigInteger envelope = get(i);
      RevealedNote revealedNote = revealedNotesList.get(j);
      BigInteger ris = new BigInteger(id);
      cheat = cheat || checkRevealed(envelope, revealedNote, ris, pubKey);
    }
    return !cheat;
  }
  

  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */
  public static EnvelopesList load(String filename) {
    return new EnvelopesList(Tools.readFile(filename));
  }
  

  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */  
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  }
  
  
  /**
   * Returns a string representation of this EnvelopesList.
   * @return a string representation of this EnvelopesList
   */  
  public String toString() {
    String result = "";
    for (int i = 0; i < v.size(); i++) {
      result += (BigInteger)v.get(i) + "\n";
    }
    return result;
  }
  

  private boolean checkRevealed(BigInteger envelope, RevealedNote revealedNote, BigInteger ris, PubKey pubKey) {
    boolean cheat = false;
    BigInteger e = pubKey.getE();
    BigInteger n = pubKey.getN();
    BigInteger amount = revealedNote.getAmount();
    BigInteger k = revealedNote.getK();
    BigInteger[] x = revealedNote.getX();
    BigInteger[] y = new BigInteger[x.length];
    for (int i = 0; i < x.length; i++) {
      if (i % 2 == 0 && !x[i].xor(x[i+1]).equals(ris)) cheat = true;
      y[i] = Tools.sha(x[i]);
    }
    BigInteger bigM = amount.shiftLeft(0);
    for (int i = 0; i < y.length; i++) {
      bigM = bigM.shiftLeft(Tools.HASH_LENGTH).or(y[i]);
    }
    BigInteger m = Tools.sha(bigM);
    BigInteger envelope2 = m.multiply(k.modPow(e, n)).mod(n);
    if (!envelope.equals(envelope2)) cheat = true;
    return cheat;
  }            
}