// Updates: 2003.06.13


package ecash.kernel;


import java.awt.*;
import java.math.*;
import java.util.*;
import java.io.*;
import java.security.*;


/**
 * This class constructs a list of full notes.
 * @author Michel Deriaz
 */  
public class FullNotesList implements Serializable {
  private Vector v;
  
  
  /**
   * Constructs a list of full notes. 
   * @param nbEnvelope the number of envelopes (the bank will then choose 
   * and sign one, and the other will have to be revealed)
   * @param amountString the amount of the notes, given as a String
   * @param id the identification of the client (bank account)
   * @param pubKey the public key of the bank
   */   
  public FullNotesList(int nbEnvelope, String amountString, String id, PubKey pubKey) {
    BigInteger amount = new BigInteger("" + (int)(Double.parseDouble(amountString) * 100.0));
    if (amount.bitLength() > Tools.HASH_LENGTH) error("amount is too big");
    BigInteger ris = new BigInteger(id);
    if (ris.bitLength() > Tools.HASH_LENGTH) error("identification is too long");    
    v = new Vector();
    for (int i = 0; i < nbEnvelope; i++) {
      v.add(createFullNote(amount, ris, pubKey));
    }
  }
      

  /**
   * Returns the specified full note.
   * @param i the index of the wanted full note
   * @return the specified full note
   */   
  public FullNote get(int i) {
    return (FullNote)v.get(i);
  }
  

  /**
   * Returns the vector containing all the FullNote objects.
   * @return the vector containing all the FullNote objects
   */   
  public Vector getValues() {
    return v;
  }
  
  
  /**
   * Creates an envelope list from the information found in the FullNote
   * objects, and returns it.
   * @return the envelopes to send to the bank, in an EnvelopesList object
   */   
  public EnvelopesList getEnvelopesList() {
    EnvelopesList envelopesList = new EnvelopesList();
    for (int i = 0; i < v.size(); i++) {
      FullNote fullNote = (FullNote)v.get(i);
      envelopesList.add(fullNote.getEnvelope());
    }
    return envelopesList;
  }

     
  private FullNote createFullNote(BigInteger amount, BigInteger ris, PubKey pubKey) {
    BigInteger e = pubKey.getE();
    BigInteger n = pubKey.getN();
    BigInteger k;
    do {
      k = new BigInteger(Const.KEY_SIZE, new SecureRandom());
    }
    while (!k.gcd(n).equals(BigInteger.ONE));   
    BigInteger[] x = new BigInteger[Const.NB_RIS*2];
    BigInteger[] y = new BigInteger[Const.NB_RIS*2];
    for (int i = 0; i < Const.NB_RIS; i++) {
      x[i*2] = new BigInteger(Tools.HASH_LENGTH, new SecureRandom());
      x[i*2+1] = ris.xor(x[i*2]);
      y[i*2] = Tools.sha(x[i*2]);
      y[i*2+1] = Tools.sha(x[i*2+1]);
    }
    BigInteger bigM = amount.shiftLeft(0);
    for (int i = 0; i < y.length; i++) {
      bigM = bigM.shiftLeft(Tools.HASH_LENGTH).or(y[i]);
    }
    BigInteger m = Tools.sha(bigM);
    BigInteger envelope = m.multiply(k.modPow(e, n)).mod(n);
    return new FullNote(bigM, m, envelope, amount, k, x, y);
  }    


  private static void error(String message) {
    Toolkit.getDefaultToolkit().beep();
    System.out.println("\nERROR: " + message);
    System.exit(0);
  }    
}