// Updates: 2003.06.13


package ecash.kernel;



import java.math.*;
import java.io.*;
import java.security.*;
import java.security.interfaces.*;


/**
 * This class represents a key ring, i.e. a public key and a private key.
 * @author Michel Deriaz
 */  
public class KeyRing implements Serializable {
  private BigInteger e;
  private BigInteger n;
  private BigInteger d;  
  

  /**
   * Creates a new key ring using the RSA algorithm.
   */   
  public KeyRing() {
    KeyPair kp = generateKey();
    e = ((RSAPublicKey)kp.getPublic()).getPublicExponent();
    n = ((RSAPublicKey)kp.getPublic()).getModulus();
    d = ((RSAPrivateKey)kp.getPrivate()).getPrivateExponent();
  }


  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */
  public KeyRing(String value) {
    String[] tockens = value.split("\n");
    e = new BigInteger(tockens[0]);
    n = new BigInteger(tockens[1]);
    d = new BigInteger(tockens[2]);
  }


  /**
   * Returns the public exponent.
   * @return e
   */ 
  public BigInteger getE() {
    return e;
  }


  /**
   * Returns the modulus.
   * @return n
   */ 
  public BigInteger getN() {
    return n;
  }
  

  /**
   * Returns the private exponent.
   * @return d
   */   
  public BigInteger getD() {
    return d;
  }
  
  
  /**
   * Creates and returns the public key.
   * @return the public key
   */   
  public PubKey getPublic() {
    return new PubKey(e, n);
  }
   

  /**
   * Returns a string representation of this KeyRing.
   * @return a string representation of this KeyRing
   */      
  public String toString() {
    return "" + e + "\n" + n + "\n" + d;
  }
  
  
  private KeyPair generateKey() { 
    KeyPairGenerator keyPairGen = null;
    try {
      keyPairGen = KeyPairGenerator.getInstance("RSA");
    }
    catch(NoSuchAlgorithmException e) {
      System.out.println(e);
      return null;
    }      
    keyPairGen.initialize(Const.KEY_SIZE, new SecureRandom());
    return keyPairGen.generateKeyPair();      
  }        
}