// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;


/**
 * Contains a bank note and its corresponding signed hash. This allows a 
 * user to check that the bank note is valid, and of course that the bank
 * signed it.
 * @author Michel Deriaz
 */
public class Note implements Serializable {
  private BigInteger bigM;
  private BigInteger signedNote;
  

  /**
   * Constructs a Note object from a bank note, called bigM, and its 
   * corresponding signed hash, called signedNote.
   * @param bigM the bank note
   * @param signedNote a signed hash of the note
   */   
  public Note(BigInteger bigM, BigInteger signedNote) {
    this.bigM = bigM;
    this.signedNote = signedNote;
  }
  

  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */
  public Note(String value) {
    String[] tockens = value.split(" ");
    bigM = new BigInteger(tockens[0]);
    signedNote = new BigInteger(tockens[1]);
  }
  
  
  /**
   * Returns the bank note.
   * @return bigM, the bank note
   */   
  public BigInteger getBigM() {
    return bigM;
  }

  
  /**
   * Returns the signed hash of the note.
   * @return signedNote, a signed hash of the note
   */   
  public BigInteger getSignedNote() {
    return signedNote;
  }
  
  
  /**
   * Checks if the signature on the note is valid, or that the hash of the 
   * signed note corresponds to the note.
   * @param pubKey the public key of the bank
   * @return a boolean indicating whether the signature is correct or not
   */   
  public boolean checkSignature(PubKey pubKey) {
    BigInteger e = pubKey.getE();
    BigInteger n = pubKey.getN();    
    return signedNote.modPow(e, n).equals(Tools.sha(bigM));
  }
  

  /**
   * Checks if the revealed part of the RIS is correct. 
   * @param challengeRIS the challenge asked by the seller
   * @param risPart the revealed part of the RIS, given by the buyer
   * @return a boolean indicating whether the RIS is correct or not
   */     
  public boolean checkRIS(BigInteger challengeRIS, RISPart risPart) {
    boolean ok = true;
    BigInteger bi = bigM.shiftLeft(0);
    BigInteger two = new BigInteger("2");
    BigInteger mask = two.pow(Tools.HASH_LENGTH).subtract(BigInteger.ONE);
    BigInteger[] y = new BigInteger[Const.NB_RIS*2];
    for (int i = y.length-1; i >= 0; i--) {
      y[i] = bi.and(mask);
      bi = bi.shiftRight(Tools.HASH_LENGTH);
    }
    BigInteger[] x = risPart.getValues();
    for (int i = 0; i < x.length; i++) {
      BigInteger y2 = Tools.sha(x[i]);
      if (challengeRIS.testBit(i)) ok = ok && y2.equals(y[i*2+1]);
      else ok = ok && y2.equals(y[i*2]);
    }          
    return ok;
  }
  
  
  /**
   * Returns the amount of the note.
   * @return the amount of the note
   */     
  public String getAmount() {
    return "" + (double)(bigM.shiftRight(Tools.HASH_LENGTH*Const.NB_RIS*2).intValue()) / 100.0;
  }     
  

  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */      
  public static Note load(String filename) {
    return new Note(Tools.readFile(filename));
  }
  

  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */  
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  }
  

  /**
   * Returns a string representation of this Note.
   * @return a string representation of this Note
   */        
  public String toString() {
    return "" + bigM + " " + signedNote;
  }
}