// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.util.*;


/**
 * This class, which represents a list of RevealedNote objects, is used by
 * buyers to reveal at the bank the secret information used to build all
 * the notes. 
 * @author Michel Deriaz
 * @see RevealedNote
 */  
public class RevealedNotesList {
  private Vector v;
  
  
  /**
   * Creates a revealed notes list from the information found in a 
   * FullNotesList object. This will be done except for the note designated
   * by the bank choice.
   * @param fullNotesList the FullNotesList
   * @param bankChoice the bank choice
   */
  public RevealedNotesList(FullNotesList fullNotesList, BankChoice bankChoice) {
    v = new Vector();
    Vector values = fullNotesList.getValues();
    for (int i = 0; i < values.size(); i++) {
      FullNote fn = (FullNote)values.get(i);
      if (i == bankChoice.toInt()) continue;
      else v.add(new RevealedNote(fn));
    }
  }
  

  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */  
  public RevealedNotesList(String value) {
    v = new Vector();
    String[] lines = value.split("\n");
    for (int i = 0; i < lines.length; i++) {
      String[] items = lines[i].split(" ");
      BigInteger amount = new BigInteger(items[0]);
      BigInteger k = new BigInteger(items[1]);
      BigInteger[] x = new BigInteger[items.length-2];
      for (int j = 0; j < x.length; j++) {
        x[j] = new BigInteger(items[j+2]);
      }
      v.add(new RevealedNote(amount, k, x));
    }
  }
  
  
  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */  
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  }
    
  
  /**
   * Returns the number of notes.
   * @return the number of notes
   */   
  public int size() {
    return v.size();
  }
  

  /**
   * Gets the revealed note designed by the specific index.
   * @return the revealed note designed by the specific index
   */     
  public RevealedNote get(int index) {
    return (RevealedNote)v.get(index);
  }
  
  
  /**
   * Returns the amount of the notes if they are all the same, "-1" 
   * otherwise.
   * @return the amount of the notes if they are all the same, "-1" 
   * otherwise
   */
  public String getAmount() {
    int amount = -1;
    for (int i = 0; i < v.size(); i++) {
      int val = ((RevealedNote)v.get(i)).getAmount().intValue();
      if (i == 0) amount = val;
      else if (val != amount) return "-1";
    }
    return "" + (double)amount / 100.0;
  }
  

  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */  
  public static RevealedNotesList load(String filename) {
    return new RevealedNotesList(Tools.readFile(filename));
  }
    
  
  /**
   * Returns a string representation of this RevealedNotesList.
   * @return a string representation of this RevealedNotesList
   */  
  public String toString() {
    String result = "";
    for (int i = 0; i < v.size(); i++) {
      RevealedNote rn = (RevealedNote)v.get(i);
      result += rn.getAmount() + " " + rn.getK() + " ";
      BigInteger[] x = rn.getX();
      for (int j = 0; j < x.length; j++) {
        result += x[j] + " ";
      }
      result += '\n';
    }
    return result;
  }
}