// Updates: 2003.06.13


package ecash.kernel;


import java.io.*;
import java.math.*;
import java.util.*;


/**
 * Contains all the notes and information obtained from buyers during
 * transactions. Calls to the add() and the remove() methods will be taken
 * into account only after a call to addCommit() and removeCommit()
 * respectively. The clear() method will initialize this process. Each item
 * of this list is a SellerPurseUnit object.
 * @author Michel Deriaz
 * @see SellerPurseUnit
 */
public class SellerPurse implements Serializable {
  private Vector v, vTmp, indexes;
  
  
  /**
   * Creates a SellerPurse.
   */    
  public SellerPurse() {
    v = new Vector();
    clear();
  }
  

  /**
   * Adds a new note and the information obtained from the buyer during the
   * transaction.
   * @param note the note
   * @param challengeRIS the challenge
   * @param risPart the revealed part of the RIS
   * @param buyerNickname the nickname provided by the buyer in order to 
   * identify the origin of the payment
   * @see #addCommit
   */     
  public void add(Note note, BigInteger challengeRIS, RISPart risPart, String buyerNickname) {
    vTmp.add(new SellerPurseUnit(note, challengeRIS, risPart, buyerNickname));
  }
  

  /**
   * Let's take into effect all the calls made to the add() method, and 
   * then calls the clear() method.
   * @see #add
   */
  public void addCommit() {
    v.addAll(vTmp);
    clear();
  }
  

  /**
   * Removes a note.
   * @param index the index of the note to remove
   * @see #removeCommit
   */  
  public void remove(int index) {
    vTmp.add(v.get(index));
  }
  

  /**
   * Let's take into effect all the calls made to the remove() method, and 
   * then calls the clear() method.
   * @see #remove
   */
  public void removeCommit() {
    v.removeAll(vTmp);
    clear();
  }  
  
    
  /**
   * Gets the specified note.
   * @param index the index in the list
   */      
  public Note getNote(int index) {
    return ((SellerPurseUnit)v.get(index)).getNote();
  }
  

  /**
   * Gets the challenge imposed to the specified note.
   * @param index the index of the specified note
   * @return the challenge imposed to the specified note
   */   
  public BigInteger getChallengeRIS(int index) {
    return ((SellerPurseUnit)v.get(index)).getChallengeRIS();
  }
  
  
  /**
   * Gets the revealed part of the RIS for the specified note.
   * @param index the index of the specified note
   * @return the revealed part of the RIS for the specified note
   */   
  public RISPart getRISPart(int index) {
    return ((SellerPurseUnit)v.get(index)).getRISPart();
  }
  
  
  /**
   * Gets the nickname of the buyer that sent the specified note.
   * @param index the index of the specified note
   * @return the nickname of the buyer that sent the specified note    
   */   
  public String getBuyerNickname(int index) {
    return ((SellerPurseUnit)v.get(index)).getBuyerNickname();
  }
    

  /**
   * Returns the list of the amounts of the notes. Each entry of this
   * Vector is a String.
   * @return a vector containing the amounts of all the notes
   */  
  public Vector getAmountList() {
    Vector result = new Vector();
    for(int i = 0; i < v.size(); i++) {
      SellerPurseUnit spu = (SellerPurseUnit)v.get(i);
      result.add("" + spu.getNote().getAmount());
    }
    return result;
  }
 

  /**
   * Returns the number of notes.
   * @return the number of notes
   */    
  public int size() {
    return v.size();
  }
  
  
  /**
   * Initializes the data for the add() and the remove() methods.
   */    
  public void clear() {
    vTmp = new Vector();
  }
  

  /**
   * Initializes the findAmount() method.
   */    
  public void initFindAmount() {
    indexes = new Vector();
  }
  

  /**
   * Returns the first index of a note with the specified amount. Then the 
   * note is "marked" in order to avoid to return the same note when the
   * method is called again with the same parameter. The initFindAmount()
   * will "unmark" all the notes.
   * @param amount the amount to find
   * @return the index in the list
   */    
  public int findAmount(String amount) {
    for (int i = 0; i < v.size(); i++) {
      if (indexes.contains("" + i)) continue;
      SellerPurseUnit spu = (SellerPurseUnit)v.get(i);
      if (spu.getNote().getAmount().equals(amount)) {
        indexes.add("" + i);
        return i;
      }
    }
    return -1;
  }
}