// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;
import java.util.*;
import java.security.*;


/**
 * This class provides some tools.
 * @author Michel Deriaz
 */
public class Tools {
  
  
  /**
   * Defines the length in bits of the result of the hash function. This 
   * package uses the sha function, so the value must be 160. This value 
   * is important because it is used for exemple when creating a note.
   */   
  public static final int HASH_LENGTH = 160;

       
  private Tools() {
  }
  
  
  /**
   * Signs an envelope using the private key of the bank.
   * @param envelope the envelope to sign
   * @param keyring the key ring of the bank, which contains the private 
   * key used to sign the note
   * @return a signed envelope
   */    
  public static BigInteger signEnvelope(BigInteger envelope, KeyRing keyRing) {
    return envelope.modPow(keyRing.getD(), keyRing.getN());
  }
  

  /**
   * Unblinds a signed envelope in order to get the signed note. This 
   * method needs a full notes list and the bank choice to get the blinding
   * factor k.
   * @param signedEnvelope the envelope signed by the bank and containing 
   * the wanted note
   * @param fullNotesList the full notes list
   * @param bankChoice the choice of the bank
   * @param pubKey the public key of the bank
   * @return the note, signed blindfully by the bank
   */ 
  public static BigInteger unblind(BigInteger signedEnvelope, FullNotesList fullNotesList, BankChoice bankChoice, PubKey pubKey) {
    BigInteger k = fullNotesList.get(bankChoice.toInt()).getK();
    BigInteger n = pubKey.getN();
    return signedEnvelope.multiply(k.modInverse(n)).mod(n);
  }
  
  
  /**
   * Computes a hash of the message, using the SHA function.
   * @param message the message to hash
   * @return a 160-bit hash of the message
   */     
  public static BigInteger sha(BigInteger message) {
    BigInteger result = null;
    try {
      MessageDigest md = MessageDigest.getInstance("SHA");
      md.update(message.toByteArray());
      result = new BigInteger(md.digest());
    }
    catch (Exception e) {
      System.out.println(e);  
    }    
    return result.abs();
  }  
  
  
  /**
   * Returns the content of a file as a String. The path of this file is
   * specified by the Const.DIR constant.
   * @param filename the filename
   * @return the content of a file as a String.
   * @see Const#DIR
   */   
  public static String readFile(String filename) {
    File f = new File(Const.DIR + filename);
    int size = (int)f.length ();
    char cbuf[] = new char[size];
    try {
      FileReader fr = new FileReader(Const.DIR + filename);    
      fr.read(cbuf);
      fr.close();
    }
    catch(IOException ioe) {
      return null;
    }
    return String.valueOf(cbuf);
  }
  

  /**
   * Writes a String in a file. The path of this file is specified by the
   * Const.DIR constant.
   * @param filename the filename
   * @param text the String to write
   * @see Const#DIR
   */    
  public static void writeFile(String filename, String text) {
    try {
      File f = new File(Const.DIR);
      if (!f.exists()) f.mkdir();
      FileWriter fw = new FileWriter(Const.DIR + filename);
      fw.write(text);
      fw.close();
    }
    catch(IOException ioe) {
      System.out.println(ioe);
    }
  }    


  /**
   * Reads an object from the specified file.
   * @param filename the filename
   * @return the object found in the specified file
   */
  public static Object readObject(String filename) {
    Object obj = null;
    try {
      FileInputStream fis = new FileInputStream(filename);
      ObjectInputStream ois = new ObjectInputStream(fis);
      obj = ois.readObject();
      fis.close();
      ois.close();
    }
    catch (IOException e) { // file doesn't exist
    }
    catch (ClassNotFoundException e) { 
      System.out.println(e);
    }
    return obj;
  }		   
  
  
  /**
   * Writes the specified object into the specified file.
   * @param filename the filename
   * @param object the object to save in a file
   */
  public static void writeObject(String filename, Object object) {
    try {
      FileOutputStream fos = new FileOutputStream(filename);
      ObjectOutputStream oos = new ObjectOutputStream(fos);
      oos.writeObject(object);
      oos.flush();
      fos.close();
      oos.close();
    }
    catch (IOException e) {
      System.out.println(e);
    }
  }
  
 
  /**
   * Transforms a vector of values (amounts) into a notes expression. Each
   * item of the vector must be a String and must represent an amount.
   * @param v the vector of amounts
   * @return the corresponding notes expression
   * @see #notesExpToValues
   */  
  public static String valuesToNotesExp(Vector v) {
    if (v == null) return null;
    v.add("-1");
    int mult = 0;
    String last = "";
    String result = "";
    for (int i = 0; i < v.size(); i++) {
      String current = (String)v.get(i);
      if (i == 0) last = current;
      if (current.equals(last)) mult++;
      else {
        if (result != "") result += " - ";
        result += last;
        if (mult > 1) result += "(" + mult + ")";
        mult = 1;
      }
      last = current;
    }
    return result;
  }
  
  
  /**
   * Splits a notes expression into a vector of values. Each item of this
   * vector is a String and represents an amount.
   * @param notesExp the notes expression to split
   * @return a vector of values (amounts)
   * @see #valuesToNotesExp
   */ 
  public static Vector notesExpToValues(String notesExp) { 
    Vector result = new Vector();
    String[] entry = notesExp.replaceAll(" ", "").split("-");
    for (int i = 0; i < entry.length; i++) {
      int sep = entry[i].indexOf("(");
      if (sep == -1) result.add("" + Double.parseDouble(entry[i]));
      else {
        String noteValue = entry[i].substring(0, sep);
        int mult = Integer.parseInt(entry[i].substring(sep + 1, entry[i].indexOf(")")));
        for (int j = 0; j < mult; j++) {
          result.add("" + Double.parseDouble(noteValue));
        }
      }
    }
    return result;
  }
  
  
  /**
   * This method rounds a double by keeping only 2 decimals (centimes).
   * @param d the double to round
   * @return a double with maximum 2 decimals
   */    
  public static double round(double d) {
    return (Math.rint(d * 100.0) / 100.0);
  }
  
  
  /**
   * This method rounds a double by keeping only 2 decimals (centimes).
   * @param s the double to round
   * @return a double with maximum 2 decimals
   */      
  public static double round(String s) {
    try {
      return round(Double.parseDouble(s));
    }
    catch (Exception e) {}
    return 0.0;
  }  
}    