// Updates: 2003.06.13


package ecash.kernel;


import java.math.*;
import java.io.*;
import java.util.*;


/**
 * A list of all the notes already returned to the bank. Every note is
 * recorded in order to avoid that people re-use several times the same
 * note. Each item of this list is a UsedNote object.
 * @author Michel Deriaz
 * @see UsedNote
 */
public class UsedNotesList implements Serializable {
  private Vector v;


  /**
   * Creates a new list of used notes.
   */ 
  public UsedNotesList() {
    v = new Vector();
  }
   
   
  /**
   * Creates a new list using the specified values.
   * @param v the values
   */    
  public UsedNotesList(Vector v) {
    this.v = v;
  }
      

  /**
   * Re-build the object from it's string representation.
   * @param value the string representation of the object
   */    
  public UsedNotesList(String value) {
    v = new Vector();
    if (value == null) return;
    String[] lines = value.split("\n");
    for (int i = 0; i < lines.length; i++) {
      String[] tockens = lines[i].split(" ");
      BigInteger bigM = new BigInteger(tockens[0]);
      BigInteger[] x = new BigInteger[tockens.length-1];
      for (int j = 1; j < tockens.length; j++) {
        x[j-1] = new BigInteger(tockens[j]);
      }
      v.add(new UsedNote(bigM, x));
    }
  }
  
 
  /**
   * Adds a new note to the list.
   * @param bigM the note
   * @param x the revealed part of the RIS
   */  
  public void add(BigInteger bigM, BigInteger[] x) {
    v.add(new UsedNote(bigM, x));
  }

  
  /**
   * Checks if a note has already been used, and if so, identifies the 
   * cheater. The returned value can be:
   * <br>0, if nobody cheats (note seen for the first time)
   * <br>-1, the seller cheats (same RIS)
   * <br>n > 0, the buyer with account n cheats (differents valid RIS)
   *
   * @param bigM the note
   * @param x the revealed part of the RIS
   * @return 0 if nobody cheats, -1 if the seller cheats, n if the buyer 
   * with account n cheats
   */
  public int check(BigInteger bigM, BigInteger[] x) {
    for (int i = 0; i < v.size(); i++) {
      UsedNote usedNote = (UsedNote)v.get(i);
      if (bigM.equals(usedNote.getBigM())) {
        BigInteger[] x2 = usedNote.getX();
        for (int j = 0; j < x.length; j++) {
          BigInteger n = x[j].xor(x2[j]);
          if (!n.equals(BigInteger.ZERO)) return n.intValue(); // n cheats
        }
        return -1; // the seller cheats
      }
    }
    return 0; // nobody cheats
  }
  

  /**
   * Creates and returns a copy of this list.
   * @return a copy of this list
   */ 
  public UsedNotesList copy() {
    return new UsedNotesList(v);
  }
      

  /**
   * Re-build the object from it's file representation.
   * @param filename the filename
   */    
  public static UsedNotesList load(String filename) {
    return new UsedNotesList(Tools.readFile(filename));
  }
  

  /**
   * Saves this object into it's file representation.
   * @param filename the filename
   */
  public void save(String filename) {
    Tools.writeFile(filename, toString());
  }  
   

  /**
   * Returns a string representation of this UsedNotesList.
   * @return a string representation of this UsedNotesList
   */         
  public String toString() {
    String result = "";
    for (int i = 0; i < v.size(); i++) {
      UsedNote usedNote = (UsedNote)v.get(i);
      result += usedNote.getBigM() + " ";
      BigInteger[] x = usedNote.getX();
      for (int j = 0; j < x.length; j++) {
        result += x[j] + " ";
      }
      result += "\n";
    }
    return result;
  }
}